'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      TaggedTemplateExpression: {
        enter: function enter(path, _ref2) {
          var opts = _ref2.opts;

          var addDisplayName = opts.displayName === undefined || opts.displayName === null ? true : opts.displayName;
          var addIdentifier = opts.ssr === undefined || opts.ssr === null ? true : opts.ssr;
          var tag = path.node.tag;

          if (!isStyled(tag)) return;
          if (path.node._styledComponentsSeen) {
            return;
          }

          var displayName = void 0;

          if (addDisplayName) {
            path.find(function (path) {
              if (path.isAssignmentExpression()) {
                displayName = path.node.left;
              } else if (path.isObjectProperty()) {
                displayName = path.node.key;
              } else if (path.isVariableDeclarator()) {
                displayName = path.node.id;
              } else if (path.isStatement()) {
                // we've hit a statement, we should stop crawling up
                return true;
              }

              // we've got an displayName! no need to continue
              if (displayName) return true;
            });

            // ensure that we have an displayName we can inherit from
            if (!displayName) return;

            // foo.bar -> bar
            if (t.isMemberExpression(displayName)) {
              displayName = displayName.property;
            }

            // identifiers are the only thing we can reliably get a name from
            if (!t.isIdentifier(displayName)) {
              displayName = undefined;
            } else {
              displayName = displayName.name;
            }
          }

          var newNode = void 0;
          if (addDisplayName && addIdentifier) {
            id++;
            newNode = buildNodeWithDisplayNameAndIdentifier({
              VALUE: path.node,
              DISPLAYNAME: t.stringLiteral(displayName),
              IDENTIFIER: t.numericLiteral(id)
            });
          } else if (addDisplayName) {
            newNode = buildNodeWithDisplayName({
              VALUE: path.node,
              DISPLAYNAME: t.stringLiteral(displayName)
            });
          } else if (addIdentifier) {
            id++;
            newNode = buildNodeWithIdentifier({
              VALUE: path.node,
              IDENTIFIER: t.numericLiteral(id)
            });
          }
          path.node._styledComponentsSeen = true;

          if (!newNode) return;
          path.replaceWith(newNode);
        }
      }
    }
  };
};

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buildNodeWithDisplayNameAndIdentifier = (0, _babelTemplate2.default)('(function() { var c = VALUE;  c.identifier = IDENTIFIER;  c.displayName = DISPLAYNAME; return c })()');
var buildNodeWithDisplayName = (0, _babelTemplate2.default)('(function() { var c = VALUE;  c.displayName = DISPLAYNAME; return c })()');
var buildNodeWithIdentifier = (0, _babelTemplate2.default)('(function() { var c = VALUE;  c.identifier = IDENTIFIER; return c })()');

var isStyled = function isStyled(tag) {
  return tag.object && tag.object.name == 'styled' || tag.callee && tag.callee.name == 'styled';
};

var id = 0;